"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _lodash = _interopRequireDefault(require("lodash.compact"));

var _child_process = _interopRequireDefault(require("child_process"));

/**
 * Parse value based on request type
 * @param {*} type
 * @param {*} value
 */
var parseType = function parseType() {
  var values = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var type = values[3].toString().toUpperCase();

  switch (type) {
    case 'SOA':
      return values.slice(4).toString().replace(/,/g, ' ');

    case 'MX':
      {
        return {
          priority: values[4],
          server: values[5]
        };
      }

    default:
      return values[values.length - 1];
  }
};

var parseSection = function parseSection(values, section) {
  if (section === 'answer' || section === 'additional') {
    return {
      domain: values[0],
      type: values[3],
      ttl: values[1],
      "class": values[2],
      value: parseType(values)
    };
  }

  return values;
};

var parse = function parse() {
  var output = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  var regex = /(;; )([^\s]+)( SECTION:)/g;
  var result = {};
  var data = output.split(/\r?\n/);
  var section = 'header';

  if (data.length < 6) {
    var msg = data[data.length - 2];

    if (!msg || msg.length <= 1) {
      msg = output;
    }

    throw new Error(msg);
  }

  data.forEach(function (line, i) {
    var m;
    var changed = false;
    if (i && !line) section = '';else {
      do {
        m = regex.exec(line);

        if (m) {
          changed = true;
          section = m[2].toLowerCase();
        }
      } while (m);
    }

    if (section) {
      if (!result[section]) result[section] = [];

      if (!changed && line) {
        if (section === 'header') result[section].push(parseSection((0, _lodash["default"])(line.split(/\t/)), section));else result[section].push(parseSection((0, _lodash["default"])(line.split(/\s+/g)), section));
      }
    }
  });
  result.time = Number(data[data.length - 6].replace(';; Query time: ', '').replace(' msec', ''));
  result.server = data[data.length - 5].replace(';; SERVER: ', '');
  result.datetime = data[data.length - 4].replace(';; WHEN: ', '');
  result.size = Number(data[data.length - 3].replace(';; MSG SIZE  rcvd: ', ''));
  return result;
};

var dig = function dig() {
  var args = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var raw = options.raw === true ? options.raw : args.includes('+short');
  var digCMD = options.dig || 'dig';
  return new Promise(function (resolve, reject) {
    var process = _child_process["default"].spawn(digCMD, args);

    var shellOutput = '';
    process.stdout.on('data', function (chunk) {
      shellOutput += chunk;
    });
    process.stdout.on('error', function (error) {
      reject(error);
    });
    process.stdout.on('end', function () {
      try {
        var result = raw !== true ? parse(shellOutput) : shellOutput.replace(/\n$/, '');
        resolve(result);
      } catch (err) {
        reject(err);
      }
    });
  });
};

var _default = dig;
exports["default"] = _default;
module.exports = dig;